/// <reference types="bluebird" />
/// <reference types="sequelize" />
import * as Bluebird from 'bluebird';
import * as Sequelize from 'sequelize';
import { I18nBundle } from 'pretur.i18n';
import { Spec } from 'pretur.spec';
import { Pool } from './pool';
import { InsertRequest, UpdateRequest, RemoveRequest } from 'pretur.sync';
export interface ResultItemAppender {
    appendError(error: I18nBundle): void;
    appendWarning(warning: I18nBundle): void;
}
export interface ErrorHandler<T> {
    (data: T, error: any, rip: ResultItemAppender): Bluebird<void>;
}
export interface Insert<T> {
    (transaction: Sequelize.Transaction, item: InsertRequest<T>, rip: ResultItemAppender, context: any): Bluebird<number | void>;
}
export interface Update<T> {
    (transaction: Sequelize.Transaction, item: UpdateRequest<T>, rip: ResultItemAppender, context: any): Bluebird<void>;
}
export interface Remove<T> {
    (transaction: Sequelize.Transaction, item: RemoveRequest<T>, rip: ResultItemAppender, context: any): Bluebird<void>;
}
export interface Synchronizer<T> {
    (transaction: Sequelize.Transaction, item: InsertRequest<T> | UpdateRequest<T> | RemoveRequest<T>, rip: ResultItemAppender, context: any): Bluebird<number | void>;
}
export interface UnitializedSynchronizer<T> {
    synchronizer: Synchronizer<T>;
    initialize(pool: Pool): void;
}
export interface SynchronizationInterceptor<T, R> {
    (transaction: Sequelize.Transaction, item: T, rip: ResultItemAppender, pool: Pool, context: any): Bluebird<R>;
}
export interface BuildSynchronizerOptions<T> {
    insertErrorHandler?: ErrorHandler<InsertRequest<T>>;
    updateErrorHandler?: ErrorHandler<UpdateRequest<T>>;
    removeErrorHandler?: ErrorHandler<RemoveRequest<T>>;
    insertInterceptor?: SynchronizationInterceptor<InsertRequest<T>, number | boolean>;
    updateInterceptor?: SynchronizationInterceptor<UpdateRequest<T>, boolean>;
    removeInterceptor?: SynchronizationInterceptor<RemoveRequest<T>, boolean>;
}
export declare function buildSynchronizer<T>(spec: Spec<T>, options?: BuildSynchronizerOptions<T>): UnitializedSynchronizer<T>;
